package com.sillycycle.bagleyd.pyraminx;

/*
 * @(#)Pyraminx.java
 *
 * Copyright 1994 - 2020  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.Color;
import java.awt.Cursor;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import javax.swing.JFrame;

/**
 * The <code>Pyraminx</code> class extends the
 * <code>PyraminxCanvas</code> for handling the responses to the user
 * for the <code>PyraminxFrame</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/puzzles.html
 */

/*
The Simple Solutions to Cubic Puzzles by James G. Nourse, Bantam Books,
New York, 1981. pp 8-15.

Mastering the Magic Pyramid by Tom Werneck, Evans Brothers Limited, London,
1981. pp 109-111.

Beyond Rubik's Cube: spheres, pyramids, dodecahedrons and God knows what
else by Douglas R. Hofstadter, Scientific American, July 1982, pp 16-31.

Magic Cubes 1996 Catalog of Dr. Christoph Bandelow.
*/

public class Pyraminx extends PyraminxCanvas implements MouseWheelListener,
		MouseListener, MouseMotionListener, KeyListener {
	private static final long serialVersionUID = 42L;

	public Pyraminx(JFrame frame,
			int size, int period,
			boolean orient, boolean sticky, boolean practice,
			boolean mono, boolean reverse, String names,
			Color c0, Color c1, Color c2, Color c3,
			Color borderc, Color fg, Color bg, int delay) {
		this.frame = frame;
		this.size = size;
		this.period = period;
		this.orient = orient;
		this.sticky = sticky;
		this.practice = practice;
		this.mono = mono;
		this.reverse = reverse;
		this.faceNames = names;
		this.faceColor[0] = c0;
		this.faceColor[1] = c1;
		this.faceColor[2] = c2;
		this.faceColor[3] = c3;
		this.currentForeground = this.borderColor = borderc;
		this.foreground = fg; this.background = bg;
		this.delay = delay;
		firstPaint = true;
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.addMouseWheelListener(this);
		this.addKeyListener(this);
		setCursor(new Cursor(Cursor.HAND_CURSOR));
	}

	public boolean processKey(char key) {
		switch (key) {
		case '>':
		case '.':
			speedUpPuzzle();
			return true;
		case '<':
		case ',':
			slowDownPuzzle();
			return true;
		case '@':
			((PyraminxFrame) frame).toggleSound();
			return true;
		case '?':
			((PyraminxFrame) frame).descriptionHelp();
			return true;
		case '!':
			((PyraminxFrame) frame).featuresHelp();
			return true;
		case '^':
			((PyraminxFrame) frame).referencesHelp();
			return true;
		case 'a':
		case 'A':
			((PyraminxFrame) frame).aboutHelp();
			return true;
		case '*':
			((PyraminxFrame) frame).showBuffer();
			return true;
		default:
		}
		if (solve != null && solve.getFlag()) {
			solve.setFlag(false);
			return false;
		}
		switch (key) {
		case '2':
			period2ModePuzzle();
			break;
		case '3':
			period3ModePuzzle();
			break;
		case 'b':
		case 'B':
			bothModePuzzle();
			break;
		case 'g':
			getPuzzle();
			break;
		case 'w':
			writePuzzle();
			break;
		case 'G':
			getPuzzleFile();
			break;
		case 'W':
			writePuzzleFile();
			break;
		case 'u':
		case 'U':
			undoPuzzle();
			break;
		case 'r':
		case 'R':
			redoPuzzle();
			break;
		case 'c':
		case 'C':
			clearPuzzle();
			break;
		case 'z':
		case 'Z':
			randomizePuzzle();
			break;
		case 'p':
		case 'P':
			practicePuzzle();
			break;
		case 's':
		case 'S':
			solvePuzzle();
			break;
		case 'o':
		case 'O':
			orientizePuzzle();
			break;
		case 'y':
		case 'Y':
			stickyPuzzle();
			break;
		case 'i':
		case 'I':
			incrementPuzzle();
			break;
		case 'd':
		case 'D':
			decrementPuzzle();
			break;
		case 'q':
		case 'Q':
		case 'x':
		case 'X':
		case '\03': /* Control-C */
			((PyraminxFrame) frame).quit();
			break;
		default:
			return false;
		}
		return true;
	}

	void processPuzzleInput(int x, int y, int direction, boolean shift,
			boolean control) {
		if (solve != null && solve.getFlag()) {
			solve.setFlag(false);
			return;
		}
		movePuzzleInput(x, y, direction, shift, control);
	}

	@Override
	public void mousePressed(MouseEvent event) {
		int modifiers = event.getModifiersEx();

		if (scanPaint || stackPaint || mouseDown)
			return;
		if (solve != null && solve.getFlag()) {
			solve.setFlag(false);
			cheat = true;
			return;
		}
		if (((modifiers & InputEvent.META_DOWN_MASK) == 0) &&
				((modifiers & InputEvent.ALT_DOWN_MASK) == 0)) {
			if (randomPaints == 0) {
				selectPuzzle((modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
					event.getX(), event.getY());
				mouseDown = true;
			}
		} else {
			if ((event.getClickCount() & 1) == 1)
				randomizePuzzleWithQuery();
			else
				randomizePuzzle();
		}
	}

	@Override
	public void mouseClicked(MouseEvent event) {
		/* Method required but keyPressed does work */
	}

	@Override
	public void mouseReleased(MouseEvent event) {
		int modifiers = event.getModifiersEx();

		if (mouseDown) { // Do not care which mouse button
			releasePuzzle((modifiers & InputEvent.SHIFT_DOWN_MASK) != 0,
				(modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
				event.getX(), event.getY());
			mouseDown = false;
		}
	}

	@Override
	public void mouseEntered(MouseEvent event) {
		currentForeground = foreground;
		framePaint = true;
		mouseDown = false;
		repaint();
	}

	@Override
	public void mouseExited(MouseEvent event) {
		if (mouseDown && currentFace >= 0) {
			releasePaint = true;
		}
		currentForeground = borderColor;
		framePaint = true;
		mouseDown = false;
		repaint();
	}

	@Override
	public void mouseDragged(MouseEvent event) {
		/* Method required but keyPressed does work */
	}

	@Override
	public void mouseMoved(MouseEvent event) {
		lastX = event.getX();
		lastY = event.getY();
	}

	@Override
	public void mouseWheelMoved(MouseWheelEvent event) {
		int notches = event.getWheelRotation();
		int modifiers = event.getModifiersEx();
		boolean shift = ((modifiers & InputEvent.SHIFT_DOWN_MASK) != 0);
		boolean control = ((modifiers & InputEvent.CTRL_DOWN_MASK) != 0);

		processPuzzleInput(lastX, lastY,
			((notches < 0) ? TOP : BOTTOM), shift, control);
	}

	@Override
	public void keyPressed(KeyEvent event) {
		int modifiers = event.getModifiersEx();
		boolean shift = ((modifiers & InputEvent.SHIFT_DOWN_MASK) != 0);
		boolean control = ((modifiers & InputEvent.CTRL_DOWN_MASK) != 0);

		switch (event.getKeyCode()) {
		case KeyEvent.VK_ESCAPE:
			((PyraminxFrame) frame).shuffleDown();
			return;
		case KeyEvent.VK_F11:
		case KeyEvent.VK_DIVIDE:
			processPuzzleInput(lastX, lastY, CCW, shift, control);
			return;
		case KeyEvent.VK_UP:
		case KeyEvent.VK_KP_UP:
		case KeyEvent.VK_NUMPAD8:
			processPuzzleInput(lastX, lastY, TOP, shift, control);
			return;
		case KeyEvent.VK_PAGE_UP:
		case KeyEvent.VK_NUMPAD9:
			processPuzzleInput(lastX, lastY, TR, shift, control);
			return;
		case KeyEvent.VK_LEFT:
		case KeyEvent.VK_KP_LEFT:
		case KeyEvent.VK_NUMPAD4:
			processPuzzleInput(lastX, lastY, LEFT, shift, control);
			return;
		case KeyEvent.VK_F12:
		case KeyEvent.VK_NUMPAD5:
		case KeyEvent.VK_CLEAR:
			processPuzzleInput(lastX, lastY, CW, shift, control);
			return;
		case KeyEvent.VK_RIGHT:
		case KeyEvent.VK_KP_RIGHT:
		case KeyEvent.VK_NUMPAD6:
			processPuzzleInput(lastX, lastY, RIGHT, shift, control);
			return;
		case KeyEvent.VK_END:
		case KeyEvent.VK_NUMPAD1:
			processPuzzleInput(lastX, lastY, BL, shift, control);
			return;
		case KeyEvent.VK_DOWN:
		case KeyEvent.VK_KP_DOWN:
		case KeyEvent.VK_NUMPAD2:
			processPuzzleInput(lastX, lastY, DOWN, shift, control);
			return;
		default:
			processKey(event.getKeyChar());
		}
	}

	@Override
	public void keyReleased(KeyEvent event) {
		/* Method required but keyPressed does work */
	}

	@Override
	public void keyTyped(KeyEvent event) {
		/* Method required but keyPressed does work */
	}
}
